@@ -1,5 +1,6 @@
 #include <algorithm>
 #include <cassert>
+#include <cstring>
 #include <opkele/util.h>
 #include <opkele/exception.h>
 #include <opkele/data.h>
@@ -7,15 +8,60 @@
 #include <openssl/sha.h>
 #include <openssl/hmac.h>
 #include <curl/curl.h>
-#include <pcre++.h>
 
 #include <iostream>
 
 #include "config.h"
 
+#include <pcre.h>
+
 namespace opkele {
     using namespace std;
 
+    class pcre_matches_t {
+	public:
+	    int *_ov;
+	    int _s;
+
+	    pcre_matches_t() : _ov(0), _s(0) { }
+	    pcre_matches_t(int s) : _ov(0), _s(s) {
+		if(_s&1) ++_s;
+		_s += _s>>1;
+		_ov = new int[_s];
+	    }
+	    ~pcre_matches_t() throw() { if(_ov) delete[] _ov; }
+
+	    int begin(int i) const { return _ov[i<<1]; }
+	    int end(int i) const { return _ov[(i<<1)+1]; }
+	    int length(int i) const { int t=i<<1; return _ov[t+1]-_ov[t]; }
+    };
+
+    class pcre_t {
+	public:
+	    pcre *_p;
+
+	    pcre_t() : _p(0) { }
+	    pcre_t(pcre *p) : _p(p) { }
+	    pcre_t(const char *re,int opts) : _p(0) {
+		static const char *errptr; static int erroffset;
+		_p = pcre_compile(re,opts,&errptr,&erroffset,NULL);
+		if(!_p)
+		    throw internal_error(OPKELE_CP_ string("Failed to compile regexp: ")+errptr);
+	    }
+	    ~pcre_t() throw() { if(_p) (*pcre_free)(_p); }
+
+	    pcre_t& operator=(pcre *p) { if(_p) (*pcre_free)(_p); _p=p; return *this; }
+
+	    operator const pcre*(void) const { return _p; }
+	    operator pcre*(void) { return _p; }
+
+	    int exec(const string& s,pcre_matches_t& m) {
+		if(!_p)
+		    throw internal_error(OPKELE_CP_ "Trying to execute absent regexp");
+		return pcre_exec(_p,NULL,s.c_str(),s.length(),0,0,m._ov,m._s);
+	    }
+    };
+
     class curl_t {
 	public:
 	    CURL *_c;
@@ -36,7 +82,7 @@ namespace opkele {
 	|| (r=curl_easy_setopt(c,CURLOPT_MAXREDIRS,5))
 	|| (r=curl_easy_setopt(c,CURLOPT_DNS_CACHE_TIMEOUT,120))
 	|| (r=curl_easy_setopt(c,CURLOPT_DNS_USE_GLOBAL_CACHE,1))
-	|| (r=curl_easy_setopt(c,CURLOPT_USERAGENT,PACKAGE_NAME"/"PACKAGE_VERSION))
+	|| (r=curl_easy_setopt(c,CURLOPT_USERAGENT,PACKAGE_NAME"/"PACKAGE_SRC_VERSION))
 	|| (r=curl_easy_setopt(c,CURLOPT_TIMEOUT,20))
 #ifdef	DISABLE_CURL_SSL_VERIFYHOST
 	|| (r=curl_easy_setopt(c,CURLOPT_SSL_VERIFYHOST,0))
@@ -85,7 +131,7 @@ namespace opkele {
 	;
 	if(r)
 	    throw exception_curl(OPKELE_CP_ "failed to curl_easy_setopt()",r);
-	if(r=curl_easy_perform(curl))
+	if( (r=curl_easy_perform(curl)) )
 	    throw exception_curl(OPKELE_CP_ "failed to curl_easy_perform()",r);
 	params_t p; p.parse_keyvalues(response);
 	if(p.has_param("assoc_type") && p.get_param("assoc_type")!="HMAC-SHA1")
@@ -245,7 +291,7 @@ namespace opkele {
 	;
 	if(r)
 	    throw exception_curl(OPKELE_CP_ "failed to curl_easy_setopt()",r);
-	if(r=curl_easy_perform(curl))
+	if( (r=curl_easy_perform(curl)) )
 	    throw exception_curl(OPKELE_CP_ "failed to curl_easy_perform()",r);
 	params_t pp; pp.parse_keyvalues(response);
 	if(pp.has_param("invalidate_handle"))
@@ -278,37 +324,60 @@ namespace opkele {
 	r = curl_easy_perform(curl);
 	if(r && r!=CURLE_WRITE_ERROR)
 	    throw exception_curl(OPKELE_CP_ "failed to curl_easy_perform()",r);
-	pcrepp::Pcre bre("<body\\b",PCRE_CASELESS);
-	// strip out everything past body
-	if(bre.search(html))
-	    html.erase(bre.get_match_start());
-	pcrepp::Pcre hdre("<head[^>]*>",PCRE_CASELESS);
-	if(!hdre.search(html))
-	    throw bad_input(OPKELE_CP_ "failed to find head");
-	html.erase(0,hdre.get_match_end()+1);
-	pcrepp::Pcre lre("<link\\b([^>]+)>",PCRE_CASELESS),
-	    rre("\\brel=['\"]([^'\"]+)['\"]",PCRE_CASELESS),
-	    hre("\\bhref=['\"]([^'\"]+)['\"]",PCRE_CASELESS);
-	while(lre.search(html)) {
-	    string attrs = lre[0];
-	    html.erase(0,lre.get_match_end()+1);
-	    if(!(rre.search(attrs)&&hre.search(attrs)))
+	static const char *re_bre = "<\\s*body\\b", *re_hdre = "<\\s*head[^>]*>",
+		     *re_lre = "<\\s*link\\b([^>]+)>",
+		     *re_rre = "\\brel\\s*=\\s*['\"]([^'\"]+)['\"]",
+		     *re_hre = "\\bhref\\s*=\\s*['\"]\\s*([^'\"\\s]+)\\s*['\"]";
+	pcre_matches_t m1(3), m2(3);
+	pcre_t bre(re_bre,PCRE_CASELESS);
+	if(bre.exec(html,m1)>0)
+	    html.erase(m1.begin(0));
+	pcre_t hdre(re_hdre,PCRE_CASELESS);
+	if(hdre.exec(html,m1)<=0)
+	    throw bad_input(OPKELE_CP_ "failed to find <head>");
+	html.erase(0,m1.end(0)+1);
+	pcre_t lre(re_lre,PCRE_CASELESS), rre(re_rre,PCRE_CASELESS), hre(re_hre,PCRE_CASELESS);
+	bool gotit = false;
+	while( (!gotit) && lre.exec(html,m1)>=2 ) {
+	    static const char *whitespace = " \t";
+	    string attrs(html,m1.begin(1),m1.length(1));
+	    html.erase(0,m1.end(0)+1);
+	    if(!( rre.exec(attrs,m1)>=2 && hre.exec(attrs,m2)>=2 ))
 		continue;
-	    if(rre[0]=="openid.server") {
-		server = hre[0];
-		if(!delegate.empty())
-		    break;
-	    }else if(rre[0]=="openid.delegate") {
-		delegate = hre[0];
-		if(!server.empty())
-		    break;
+	    string rels(attrs,m1.begin(1),m1.length(1));
+	    for(string::size_type ns = rels.find_first_not_of(whitespace);
+		    ns!=string::npos;
+		    ns=rels.find_first_not_of(whitespace,ns)) {
+		string::size_type s = rels.find_first_of(whitespace,ns);
+		string rel;
+		if(s==string::npos) {
+		    rel.assign(rels,ns,string::npos);
+		    ns=string::npos;
+		}else{
+		    rel.assign(rels,ns,s-ns);
+		    ns=s;
+		}
+		if(rel=="openid.server") {
+		    server.assign(attrs,m2.begin(1),m2.length(1));
+		    if(!delegate.empty()) {
+			gotit = true;
+			break;
+		    }
+		}else if(rel=="openid.delegate") {
+		    delegate.assign(attrs,m2.begin(1),m2.length(1));
+		    if(!server.empty()) {
+			gotit = true;
+			break;
+		    }
+		}
+		if(ns==string::npos) break;
 	    }
 	}
 	if(server.empty())
 	    throw failed_assertion(OPKELE_CP_ "The location has no openid.server declaration");
     }
 
-    assoc_t consumer_t::find_assoc(const string& server) {
+    assoc_t consumer_t::find_assoc(const string& /* server */) {
 	throw failed_lookup(OPKELE_CP_ "no find_assoc() provided");
     }
 
